/**
 * Wraps promises that may never be awaited (as their result and failure is no-longer of interest).
 *
 * It's OK that a failure is "no-longer of interest": the standard library also does this in places. Consider the
 * following code: 'Promise.race([fastResolve, slowReject])'. Here the 'slowReject' promise is no-longer of interest,
 * and its failure is essentially swallowed by the standard promise library. As long as your code is explicit when it
 * neglects promises, either with a 'mayNotAwait', 'Promise.race' or other, then there is no problem.
 */
export declare function mayAbandon<T>(promise: Promise<T>): Promise<T>;
