(function webpackUniversalModuleDefinition(root, factory) {
	if(typeof exports === 'object' && typeof module === 'object')
		module.exports = factory();
	else if(typeof define === 'function' && define.amd)
		define([], factory);
	else {
		var a = factory();
		for(var i in a) (typeof exports === 'object' ? exports : root)[i] = a[i];
	}
})(global, () => {
return /******/ (() => { // webpackBootstrap
/******/ 	"use strict";
/******/ 	var __webpack_modules__ = ({

/***/ "./src/Transformer.ts":
/*!****************************!*\
  !*** ./src/Transformer.ts ***!
  \****************************/
/***/ ((__unused_webpack_module, exports, __webpack_require__) => {



Object.defineProperty(exports, "__esModule", ({
  value: true
}));
exports.Transformer = void 0;
const upload_plugin_sdk_1 = __webpack_require__(/*! upload-plugin-sdk */ "upload-plugin-sdk");
const SupportedOutputFormat_1 = __webpack_require__(/*! upload-image-plugin/params/SupportedOutputFormat */ "./src/params/SupportedOutputFormat.ts");
const fs_1 = __webpack_require__(/*! fs */ "fs");
const PageRange_1 = __webpack_require__(/*! upload-image-plugin/params/PageRange */ "./src/params/PageRange.ts");
class Transformer {
  constructor(pipelineLintingService, pipelineMergingService, pipelineExecutionService, downloadService, eagerWorkService) {
    this.pipelineLintingService = pipelineLintingService;
    this.pipelineMergingService = pipelineMergingService;
    this.pipelineExecutionService = pipelineExecutionService;
    this.downloadService = downloadService;
    this.eagerWorkService = eagerWorkService;
  }
  async estimate(params, resolvePath, download, getMetadata) {
    const inputImage = await this.getInputImage(params, resolvePath, download, getMetadata);
    // Goes after 'getInputImage' as requires access to the pipeline's file resources (font files, etc.)
    const eagerWork = this.eagerWorkService.beginEagerWork(inputImage);
    return {
      input: inputImage,
      eagerWork,
      estimation: undefined
    };
  }
  async run(resolvePath, setMetadata, {
    input,
    eagerWork
  }) {
    const outputArtifact = "/";
    let outputFormat;
    try {
      outputFormat = await this.transformImage(input, outputArtifact, resolvePath, eagerWork);
    } catch (e) {
      if (e instanceof upload_plugin_sdk_1.TransformationDisplayableError) {
        throw e;
      }
      const page = input.pipeline.pages?.page;
      const pageCount = input.pipeline.pages?.pageCount;
      if (e.message.includes("page")) {
        if (page !== undefined && page > 0 && pageCount !== undefined && pageCount > 1) {
          throw new upload_plugin_sdk_1.TransformationDisplayableError(`Unable to process image. The page number (${page}) and/or page count (${pageCount}) parameters may be too high. Remember: page numbers are zero-based.`, "unable_to_process_image_page", true);
        }
        if (page !== undefined && page > 0) {
          throw new upload_plugin_sdk_1.TransformationDisplayableError(`Unable to process image. The page number parameter (${page}) may be too high. Remember: page numbers are zero-based.`, "unable_to_process_image_page", true);
        }
        if (pageCount !== undefined && pageCount > 0) {
          throw new upload_plugin_sdk_1.TransformationDisplayableError(`Unable to process image. The page count parameter (${pageCount}) may be too high.`, "unable_to_process_image_page", true);
        }
      }
      console.error("Unable to transform image", e);
      throw new upload_plugin_sdk_1.TransformationDisplayableError("Unable to process image. Please check the input file is a valid image and is not corrupt. If the problem persists, the image may be using an unsupported codec, or invalid parameters have been used to process the image.", "unable_to_process_image", true, 400, {
        help: "https://upload.io/docs/image-processing-api"
      });
    }
    await Promise.all([this.deleteTemporaryFiles(input), this.setContentType(outputFormat.type, outputArtifact, setMetadata)]);
  }
  async getInputImage(params, resolvePath, download, getMetadata) {
    const inputPath = resolvePath("/");
    const [pipeline, metadata, paramsFromFile] = await this.getPipeline(inputPath, params, getMetadata);
    const downloadRequests = this.downloadService.makeDownloadRequestsForPipeline(pipeline, resolvePath);
    await this.downloadService.prepareTempDirForDownloadRequests(downloadRequests);
    if (paramsFromFile === undefined) {
      return {
        path: inputPath,
        contentType: metadata.contentType,
        pipeline,
        tempDownloadedFiles: this.downloadService.attachDownloadRequestsToResponses(downloadRequests, await download(downloadRequests))
      };
    }
    const destinationFile = inputPath;
    const request = paramsFromFile.input === undefined ? {
      destinationFile,
      sourceFilePath: paramsFromFile.inputPath
    } : {
      destinationFile,
      sourceFileId: paramsFromFile.input
    };
    const downloads = await download({
      file: request,
      ...downloadRequests
    });
    const {
      file,
      ...tempFileDownloadResponses
    } = downloads;
    if (file.isError) {
      throw new Error(`Failed to download file for cropping: ${paramsFromFile.input ?? paramsFromFile.inputPath ?? "?"}`);
    }
    return {
      path: request.destinationFile,
      contentType: file.headers["content-type"],
      pipeline,
      tempDownloadedFiles: this.downloadService.attachDownloadRequestsToResponses(downloadRequests, tempFileDownloadResponses)
    };
  }
  async getPipeline(inputPath, params, getMetadata) {
    const metadata = await getMetadata("/");
    if (metadata.contentType !== "application/json") {
      return [params.pipeline, metadata, undefined];
    }
    try {
      const paramsFromFile = JSON.parse((await fs_1.promises.readFile(inputPath)).toString());
      return [this.pipelineMergingService.mergePipelines(paramsFromFile, params), metadata, paramsFromFile];
    } catch (e) {
      throw new upload_plugin_sdk_1.TransformationDisplayableError("Unable to process image. The provided file is a JSON file, but does not contain valid image processing instructions.", "unable_to_process_image_json", true, 400, {
        help: "https://upload.io/docs/image-processing-api"
      });
    }
  }
  async transformImage(input, output, resolvePath, eagerWork) {
    const outputResolved = resolvePath(output);
    const steps = this.pipelineLintingService.lintPipeline(input.pipeline.steps);
    const pages = input.pipeline.pages ?? PageRange_1.PageRange.defaultValue;
    const {
      outputFormat: outputFormatMaybe
    } = input.pipeline;
    const outputFormat = SupportedOutputFormat_1.SupportedOutputFormat.toObject(outputFormatMaybe ?? this.inferOutputFormat(input));
    const image = await this.pipelineExecutionService.executePipeline(input, steps, outputFormat, pages, eagerWork);
    // Buffer to new file, since input file and output file may be the same location, and AFAIK libvips may stream
    // its processing such that the output is written whilst the input is read.
    const tempPath = `${outputResolved}.tmp`;
    await image.toFile(tempPath);
    await fs_1.promises.rename(tempPath, outputResolved);
    return outputFormat;
  }
  inferOutputFormat({
    contentType: contentTypeFromInput
  }) {
    const contentTypeLower = contentTypeFromInput?.toLowerCase();
    const outputFormatMaybe = SupportedOutputFormat_1.SupportedOutputFormat.contentTypes.find(([, contentType]) => contentType === contentTypeLower);
    // Convert to a supported output format (e.g. HEIC, SVG, etc. will be automatically converted to JPG).
    return outputFormatMaybe?.[0] ?? "jpg";
  }
  /**
   * Delete temp files, else they will be uploaded as artifacts of the transformation.
   */
  async deleteTemporaryFiles(input) {
    await Promise.all(Object.values(input.tempDownloadedFiles).map(async x => {
      if (x !== undefined) {
        await fs_1.promises.unlink(x.request.destinationFile);
      }
    }));
  }
  async setContentType(outputFormat, output, setMetadata) {
    const mimeType = SupportedOutputFormat_1.SupportedOutputFormat.contentTypeReverseMap[outputFormat];
    await setMetadata(output, {
      contentType: mimeType
    });
  }
}
exports.Transformer = Transformer;

/***/ }),

/***/ "./src/common/PromiseUtils.ts":
/*!************************************!*\
  !*** ./src/common/PromiseUtils.ts ***!
  \************************************/
/***/ ((__unused_webpack_module, exports) => {



Object.defineProperty(exports, "__esModule", ({
  value: true
}));
exports.mayAbandon = void 0;
/**
 * Wraps promises that may never be awaited (as their result and failure is no-longer of interest).
 *
 * It's OK that a failure is "no-longer of interest": the standard library also does this in places. Consider the
 * following code: 'Promise.race([fastResolve, slowReject])'. Here the 'slowReject' promise is no-longer of interest,
 * and its failure is essentially swallowed by the standard promise library. As long as your code is explicit when it
 * neglects promises, either with a 'mayNotAwait', 'Promise.race' or other, then there is no problem.
 */
// eslint-disable-next-line @typescript-eslint/promise-function-async
function mayAbandon(promise) {
  promise.then(() => {}, () => {});
  return promise;
}
exports.mayAbandon = mayAbandon;

/***/ }),

/***/ "./src/common/TypeUtils.ts":
/*!*********************************!*\
  !*** ./src/common/TypeUtils.ts ***!
  \*********************************/
/***/ ((__unused_webpack_module, exports) => {



Object.defineProperty(exports, "__esModule", ({
  value: true
}));
exports.assertUnreachable = exports.assertUnreachableNoError = void 0;
function assertUnreachableNoError(_x) {}
exports.assertUnreachableNoError = assertUnreachableNoError;
function assertUnreachable(x) {
  throw new Error(`Didn't expect to get here: ${JSON.stringify(x)}`);
}
exports.assertUnreachable = assertUnreachable;

/***/ }),

/***/ "./src/index.ts":
/*!**********************!*\
  !*** ./src/index.ts ***!
  \**********************/
/***/ (function(__unused_webpack_module, exports, __webpack_require__) {



var __importDefault = this && this.__importDefault || function (mod) {
  return mod && mod.__esModule ? mod : {
    "default": mod
  };
};
Object.defineProperty(exports, "__esModule", ({
  value: true
}));
const upload_plugin_sdk_1 = __webpack_require__(/*! upload-plugin-sdk */ "upload-plugin-sdk");
const Transformer_1 = __webpack_require__(/*! upload-image-plugin/Transformer */ "./src/Transformer.ts");
const PipelineLintingService_1 = __webpack_require__(/*! upload-image-plugin/services/PipelineLintingService */ "./src/services/PipelineLintingService.ts");
const LayerService_1 = __webpack_require__(/*! upload-image-plugin/services/LayerService */ "./src/services/LayerService.ts");
const PipelineMergingService_1 = __webpack_require__(/*! upload-image-plugin/services/PipelineMergingService */ "./src/services/PipelineMergingService.ts");
const MathService_1 = __webpack_require__(/*! upload-image-plugin/services/MathService */ "./src/services/MathService.ts");
const DownloadService_1 = __webpack_require__(/*! upload-image-plugin/services/DownloadService */ "./src/services/DownloadService.ts");
const PipelineExecutionService_1 = __webpack_require__(/*! upload-image-plugin/services/PipelineExecutionService */ "./src/services/PipelineExecutionService.ts");
const sharp_1 = __importDefault(__webpack_require__(/*! sharp */ "sharp"));
const EagerWorkService_1 = __webpack_require__(/*! upload-image-plugin/services/EagerWorkService */ "./src/services/EagerWorkService.ts");
const FontRenamingService_1 = __webpack_require__(/*! upload-image-plugin/services/FontRenamingService */ "./src/services/FontRenamingService.ts");
const FontCachingService_1 = __webpack_require__(/*! upload-image-plugin/services/FontCachingService */ "./src/services/FontCachingService.ts");
sharp_1.default.cache({
  files: 0 // Despite clearing '/tmp' on each Lambda run, if we don't set this, our Lambdas run out of disk space after several invocations of large images.
});

const globalSharpOptions = {
  failOn: "none",
  limitInputPixels: false // Some users exceed the default limit (e.g. some paying customers use 24120×29697 images). We use our memory estimation phase to limit images instead.
};

const pipelineLintingService = new PipelineLintingService_1.PipelineLintingService();
const pipelineMergingService = new PipelineMergingService_1.PipelineMergingService();
const downloadService = new DownloadService_1.DownloadService();
const layerService = new LayerService_1.LayerService(downloadService, globalSharpOptions);
const mathService = new MathService_1.MathService();
const fontRenamingService = new FontRenamingService_1.FontRenamingService();
const fontCachingService = new FontCachingService_1.FontCachingService(fontRenamingService);
const eagerWorkService = new EagerWorkService_1.EagerWorkService(downloadService, fontCachingService);
const pipelineExecutionService = new PipelineExecutionService_1.PipelineExecutionService(layerService, mathService, globalSharpOptions);
const transformer = new Transformer_1.Transformer(pipelineLintingService, pipelineMergingService, pipelineExecutionService, downloadService, eagerWorkService);
exports["default"] = (0, upload_plugin_sdk_1.transform)({
  estimate: async ({
    params,
    resolve,
    download,
    getMetadata
  }) => await transformer.estimate(params, resolve, download, getMetadata),
  run: async ({
    resolve,
    setMetadata,
    estimationResult
  }) => await transformer.run(resolve, setMetadata, estimationResult)
});

/***/ }),

/***/ "./src/model/PaddingDefinition.ts":
/*!****************************************!*\
  !*** ./src/model/PaddingDefinition.ts ***!
  \****************************************/
/***/ ((__unused_webpack_module, exports) => {



Object.defineProperty(exports, "__esModule", ({
  value: true
}));
exports.PaddingDefinition = void 0;
var PaddingDefinition;
(function (PaddingDefinition) {
  function fromLayerStep({
    padding,
    paddingY,
    paddingX,
    paddingBottom,
    paddingLeft,
    paddingRight,
    paddingTop
  }) {
    return {
      all: padding,
      y: paddingY,
      x: paddingX,
      bottom: paddingBottom,
      top: paddingTop,
      left: paddingLeft,
      right: paddingRight
    };
  }
  PaddingDefinition.fromLayerStep = fromLayerStep;
  function fromExtendStep({
    extend,
    extendY,
    extendX,
    extendBottom,
    extendTop,
    extendLeft,
    extendRight
  }) {
    return {
      all: extend,
      y: extendY,
      x: extendX,
      bottom: extendBottom,
      top: extendTop,
      left: extendLeft,
      right: extendRight
    };
  }
  PaddingDefinition.fromExtendStep = fromExtendStep;
  function getPadding({
    all,
    y,
    x,
    bottom,
    top,
    left,
    right
  }) {
    return {
      top: top ?? y ?? all ?? 0,
      bottom: bottom ?? y ?? all ?? 0,
      left: left ?? x ?? all ?? 0,
      right: right ?? x ?? all ?? 0
    };
  }
  PaddingDefinition.getPadding = getPadding;
})(PaddingDefinition = exports.PaddingDefinition || (exports.PaddingDefinition = {}));

/***/ }),

/***/ "./src/params/Compression.ts":
/*!***********************************!*\
  !*** ./src/params/Compression.ts ***!
  \***********************************/
/***/ ((__unused_webpack_module, exports) => {



Object.defineProperty(exports, "__esModule", ({
  value: true
}));
exports.Compression = void 0;
var Compression;
(function (Compression) {
  Compression.defaultValue = 60; // Sync with: upload.config.json
})(Compression = exports.Compression || (exports.Compression = {}));

/***/ }),

/***/ "./src/params/ImagePipelineMergeBehaviour.ts":
/*!***************************************************!*\
  !*** ./src/params/ImagePipelineMergeBehaviour.ts ***!
  \***************************************************/
/***/ ((__unused_webpack_module, exports) => {



Object.defineProperty(exports, "__esModule", ({
  value: true
}));
exports.ImagePipelineMergeBehaviour = void 0;
var ImagePipelineMergeBehaviour;
(function (ImagePipelineMergeBehaviour) {
  /**
   * The default merge behaviour (i.e. if none is specified in the transformation) is to allow the user to define crops,
   * but nothing else.
   */
  ImagePipelineMergeBehaviour.defaultValue = {
    compression: "master",
    outputFormat: "master",
    outputQuality: "master",
    steps: {
      fileWhitelist: ["crop"],
      mergeBehaviour: {
        startWith: "file",
        removeDuplicates: "end"
      }
    }
  };
})(ImagePipelineMergeBehaviour = exports.ImagePipelineMergeBehaviour || (exports.ImagePipelineMergeBehaviour = {}));

/***/ }),

/***/ "./src/params/LayerBlendMode.ts":
/*!**************************************!*\
  !*** ./src/params/LayerBlendMode.ts ***!
  \**************************************/
/***/ ((__unused_webpack_module, exports) => {



Object.defineProperty(exports, "__esModule", ({
  value: true
}));
exports.LayerBlendMode = void 0;
var LayerBlendMode;
(function (LayerBlendMode) {
  function toString(format) {
    if (typeof format === "string") {
      return format;
    }
    return format.type;
  }
  LayerBlendMode.toString = toString;
})(LayerBlendMode = exports.LayerBlendMode || (exports.LayerBlendMode = {}));

/***/ }),

/***/ "./src/params/LayerGravityMode.ts":
/*!****************************************!*\
  !*** ./src/params/LayerGravityMode.ts ***!
  \****************************************/
/***/ ((__unused_webpack_module, exports) => {



Object.defineProperty(exports, "__esModule", ({
  value: true
}));
exports.LayerGravityMode = void 0;
var LayerGravityMode;
(function (LayerGravityMode) {
  function toString(format) {
    if (typeof format === "string") {
      return format;
    }
    return format.type;
  }
  LayerGravityMode.toString = toString;
})(LayerGravityMode = exports.LayerGravityMode || (exports.LayerGravityMode = {}));

/***/ }),

/***/ "./src/params/OutputQuality.ts":
/*!*************************************!*\
  !*** ./src/params/OutputQuality.ts ***!
  \*************************************/
/***/ ((__unused_webpack_module, exports) => {



Object.defineProperty(exports, "__esModule", ({
  value: true
}));
exports.OutputQuality = void 0;
var OutputQuality;
(function (OutputQuality) {
  OutputQuality.defaultValue = 80; // Sync with: upload.config.json
})(OutputQuality = exports.OutputQuality || (exports.OutputQuality = {}));

/***/ }),

/***/ "./src/params/PageRange.ts":
/*!*********************************!*\
  !*** ./src/params/PageRange.ts ***!
  \*********************************/
/***/ ((__unused_webpack_module, exports) => {



Object.defineProperty(exports, "__esModule", ({
  value: true
}));
exports.PageRange = void 0;
var PageRange;
(function (PageRange) {
  PageRange.defaultValue = {
    page: undefined,
    pageCount: undefined
  };
})(PageRange = exports.PageRange || (exports.PageRange = {}));

/***/ }),

/***/ "./src/params/SupportedOutputFormat.ts":
/*!*********************************************!*\
  !*** ./src/params/SupportedOutputFormat.ts ***!
  \*********************************************/
/***/ ((__unused_webpack_module, exports) => {



Object.defineProperty(exports, "__esModule", ({
  value: true
}));
exports.SupportedOutputFormat = void 0;
var SupportedOutputFormat;
(function (SupportedOutputFormat) {
  /**
   * Keyed by format, to make sure the compiler ensures we specify a mime type for all supported formats.
   */
  SupportedOutputFormat.contentTypeReverseMap = {
    gif: "image/gif",
    jpeg: "image/jpeg",
    jpg: "image/jpeg",
    png: "image/png",
    webp: "image/webp",
    avif: "image/avif"
  };
  SupportedOutputFormat.defaultOutputFormatSettings = {
    gif: {
      type: "gif"
    },
    jpeg: {
      type: "jpeg"
    },
    jpg: {
      type: "jpg"
    },
    png: {
      type: "png"
    },
    webp: {
      type: "webp"
    },
    avif: {
      type: "avif"
    }
  };
  function toObject(format) {
    if (typeof format === "object") {
      return format;
    }
    return SupportedOutputFormat.defaultOutputFormatSettings[format];
  }
  SupportedOutputFormat.toObject = toObject;
  SupportedOutputFormat.contentTypes = Object.entries(SupportedOutputFormat.contentTypeReverseMap);
})(SupportedOutputFormat = exports.SupportedOutputFormat || (exports.SupportedOutputFormat = {}));

/***/ }),

/***/ "./src/services/DownloadService.ts":
/*!*****************************************!*\
  !*** ./src/services/DownloadService.ts ***!
  \*****************************************/
/***/ (function(__unused_webpack_module, exports, __webpack_require__) {



var __importDefault = this && this.__importDefault || function (mod) {
  return mod && mod.__esModule ? mod : {
    "default": mod
  };
};
Object.defineProperty(exports, "__esModule", ({
  value: true
}));
exports.DownloadService = void 0;
const TypeUtils_1 = __webpack_require__(/*! upload-image-plugin/common/TypeUtils */ "./src/common/TypeUtils.ts");
const path_1 = __importDefault(__webpack_require__(/*! path */ "path"));
const fs_1 = __webpack_require__(/*! fs */ "fs");
class DownloadService {
  /**
   * Key is the file ID or file URL used by the transformation steps.
   * Files will be deleted after the transformation runs.
   */
  makeDownloadRequestsForPipeline(pipeline, resolvePath) {
    return Object.fromEntries(pipeline.steps.flatMap(x => {
      switch (x.type) {
        case "resize":
        case "crop":
        case "rotate":
        case "tint":
        case "flip":
        case "blur":
        case "sharpen":
        case "modulate":
        case "negative":
        case "extend":
        case "grayscale":
        case "flush":
          return [];
        case "text":
          {
            const {
              fontFilePath
            } = x;
            if (fontFilePath !== undefined) {
              const localFileName = this.makeLocalFileName(fontFilePath);
              return [[localFileName, {
                destinationFile: resolvePath(`/tmp/${localFileName}`),
                sourceFilePath: fontFilePath
              }]];
            }
            return [];
          }
        case "composite":
        case "layer":
          {
            const localFileName = this.makeLocalFileNameForLayerFileSource(x);
            return [[localFileName, {
              destinationFile: resolvePath(`/tmp/${localFileName}`),
              sourceFilePath: x.imageFilePath
            }]];
          }
        default:
          (0, TypeUtils_1.assertUnreachable)(x);
      }
    }));
  }
  async prepareTempDirForDownloadRequests(downloadRequests) {
    const tempFileDirectories = Array.from(new Set(Object.values(downloadRequests).map(x => path_1.default.dirname(x.destinationFile))));
    await Promise.all(tempFileDirectories.map(async x => await fs_1.promises.mkdir(x, {
      recursive: true
    })));
  }
  attachDownloadRequestsToResponses(requests, responses) {
    return Object.fromEntries(Object.entries(requests).map(([key, request]) => [key, {
      request,
      response: responses[key]
    }]));
  }
  makeLocalFileNameForLayerFileSource(step) {
    return this.makeLocalFileName(step.imageFilePath);
  }
  /**
   * Makes a file name from a string, ensuring the returned result contains no "/".
   */
  makeLocalFileName(unsafeValue) {
    return Buffer.from(unsafeValue).toString("base64");
  }
  getTempDownloadedFile(tempDownloadedFiles, safeDownloadKey) {
    const tempDownloadedFile = tempDownloadedFiles[safeDownloadKey];
    if (tempDownloadedFile === undefined) {
      throw new Error(`File with download key never downloaded: ${safeDownloadKey}`);
    }
    if (tempDownloadedFile.response.isError) {
      throw new Error(`Failed to download ancillary file for image transformations with download key: ${safeDownloadKey}`);
    }
    return tempDownloadedFile.request.destinationFile;
  }
}
exports.DownloadService = DownloadService;

/***/ }),

/***/ "./src/services/EagerWorkService.ts":
/*!******************************************!*\
  !*** ./src/services/EagerWorkService.ts ***!
  \******************************************/
/***/ ((__unused_webpack_module, exports, __webpack_require__) => {



Object.defineProperty(exports, "__esModule", ({
  value: true
}));
exports.EagerWorkService = void 0;
const ramda_1 = __webpack_require__(/*! ramda */ "ramda");
const PromiseUtils_1 = __webpack_require__(/*! upload-image-plugin/common/PromiseUtils */ "./src/common/PromiseUtils.ts");
class EagerWorkService {
  constructor(downloadService, fontCachingService) {
    this.downloadService = downloadService;
    this.fontCachingService = fontCachingService;
  }
  /**
   * Starts async work & prepares each promise for potential abandonment.
   *
   * IMPORTANT: this step is done during the estimation phase, so this work may never be used (if the platform deems the image is too large).
   */
  beginEagerWork(input) {
    return {
      fontFamilies: this.getFontFamilies(input)
    };
  }
  getFontFamilies(input) {
    const fontFilePaths = (0, ramda_1.uniq)(input.pipeline.steps.flatMap(x => {
      if (x.type !== "text" || x.fontFilePath === undefined) {
        return [];
      }
      return [x.fontFilePath];
    }));
    return Object.fromEntries(fontFilePaths.map(fontFilePath => {
      return [fontFilePath, (0, PromiseUtils_1.mayAbandon)(this.installCustomFont(input, fontFilePath))];
    }));
  }
  async installCustomFont(input, fontFilePath) {
    const localFilePath = this.getFontFileLocalPath(input, fontFilePath);
    const fontType = this.getFontType(fontFilePath);
    return await this.fontCachingService.installCustomFont(localFilePath, fontType);
  }
  getFontFileLocalPath(input, fontFilePath) {
    return this.downloadService.getTempDownloadedFile(input.tempDownloadedFiles, this.downloadService.makeLocalFileName(fontFilePath));
  }
  getFontType(fontFilePath) {
    const pathLower = fontFilePath.toLowerCase();
    if (pathLower.endsWith(".ttf")) {
      return "ttf";
    }
    if (pathLower.endsWith(".otf")) {
      return "otf";
    }
    if (pathLower.endsWith(".eot")) {
      return "eot";
    }
    if (pathLower.endsWith(".woff")) {
      return "woff";
    }
    if (pathLower.endsWith(".woff2")) {
      return "woff2";
    }
    if (pathLower.endsWith(".svg")) {
      return "svg";
    }
    return "ttf";
  }
}
exports.EagerWorkService = EagerWorkService;

/***/ }),

/***/ "./src/services/FontCachingService.ts":
/*!********************************************!*\
  !*** ./src/services/FontCachingService.ts ***!
  \********************************************/
/***/ (function(__unused_webpack_module, exports, __webpack_require__) {



var __createBinding = this && this.__createBinding || (Object.create ? function (o, m, k, k2) {
  if (k2 === undefined) k2 = k;
  Object.defineProperty(o, k2, {
    enumerable: true,
    get: function () {
      return m[k];
    }
  });
} : function (o, m, k, k2) {
  if (k2 === undefined) k2 = k;
  o[k2] = m[k];
});
var __setModuleDefault = this && this.__setModuleDefault || (Object.create ? function (o, v) {
  Object.defineProperty(o, "default", {
    enumerable: true,
    value: v
  });
} : function (o, v) {
  o["default"] = v;
});
var __importStar = this && this.__importStar || function (mod) {
  if (mod && mod.__esModule) return mod;
  var result = {};
  if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
  __setModuleDefault(result, mod);
  return result;
};
var __importDefault = this && this.__importDefault || function (mod) {
  return mod && mod.__esModule ? mod : {
    "default": mod
  };
};
Object.defineProperty(exports, "__esModule", ({
  value: true
}));
exports.FontCachingService = void 0;
const fs_1 = __webpack_require__(/*! fs */ "fs");
const path_1 = __importDefault(__webpack_require__(/*! path */ "path"));
const crypto = __importStar(__webpack_require__(/*! crypto */ "crypto"));
const upload_plugin_sdk_1 = __webpack_require__(/*! upload-plugin-sdk */ "upload-plugin-sdk");
class FontCachingService {
  constructor(fontRenamingService) {
    this.fontRenamingService = fontRenamingService;
    /**
     * We limit font sizes, as we cache them permanently in the Lambda's temp dir.
     * @private
     */
    this.maxFontSizeBytes = 1024 ** 2;
    this.maxFontCacheSizeBytes = this.maxFontSizeBytes * 20;
    /**
     * Key: hash of the original font file
     */
    this.cache = {};
    this.cacheSize = 0;
    // Must be a global folder, not a per-invocation folder (see big comment below).
    // We use a folder that isn't deleted between invocations, as we want this cache to persist for performance reasons.
    this.fontDir = "/tmp";
  }
  /**
   * Inspects the 'localFontPath' and determines if the font has been seen before, based on the file's hash.
   * - If the font HAS been seen before, returns an existing cached font file for it.
   * - If the font HAS NOT been seen before, then generates a reusable font file for it (that can be reused by
   *   subsequent transformations), and returns it.
   *
   * We do this because libvips/pango/cairo maintain an in-process cache fonts based off their embedded font name (not
   * the file name), so if we give libvips a font file path, delete it, then provide a different font path (for the same
   * font), then Pango will error, since it will attempt to load the font using its original file path, which no-longer
   * exists. The simplest workaround is to rename (the font family) of each font file before it's used. However, this
   * will cause the internal font cache inside libvips/pango to grow after each transformation, essentially resulting in
   * a memory leak. The other solution is to store the font files in a global directory (that is shared between
   * invocations), but this introduces a security issue whereby malicious users can install bad fonts that will be used
   * by other users. The best solution is to combine the approaches: store the font files in a global directory, but
   * rename the font families to use file's hash, preventing malicious users from installing bad fonts for other users.
   *
   * This method does exactly that (it installs the given font to a global font dir used by all transformations, and
   * renames the font's font family to the file's hash).
   */
  async installCustomFont(tempFontFilePath, fontType) {
    const originalFontData = await fs_1.promises.readFile(tempFontFilePath);
    const fileSize = originalFontData.length;
    if (fileSize > this.maxFontSizeBytes) {
      throw new upload_plugin_sdk_1.TransformationDisplayableError(`Custom fonts must be <=${this.maxFontSizeBytes / 1024 ** 2}MB`, "font_file_too_large", true);
    }
    const fileHash = this.getFileHash(originalFontData);
    const cached = this.cache[fileHash];
    if (cached !== undefined) {
      console.log(`Using cached font: ${cached.fontName}`);
      return cached;
    }
    await this.freeUpCache(fileSize);
    const newFont = await this.installNewFont(originalFontData, fileHash, fontType);
    if (newFont !== undefined) {
      this.addToCache(fileHash, newFont);
    }
    return newFont;
  }
  async installNewFont(originalFontData, fileHash, fontType) {
    try {
      const newFontName = fileHash;
      const newFontFilePath = path_1.default.join(this.fontDir, `font-${fileHash}`);
      const localFileSize = originalFontData.length;
      // For now, we always assume
      const newFontData = this.fontRenamingService.rename(originalFontData, fontType, newFontName);
      await fs_1.promises.writeFile(newFontFilePath, newFontData);
      return {
        fontName: newFontName,
        localFilePath: newFontFilePath,
        localFileSize
      };
    } catch (e) {
      console.warn("Unable to install custom font.", e);
      return undefined;
    }
  }
  addToCache(fileHash, newFont) {
    this.cache[fileHash] = newFont;
    this.cacheSize += newFont.localFileSize;
    console.log(`Added cached font: ${newFont.fontName}`);
  }
  async freeUpCache(requiredBytes) {
    const hasFreeSpace = () => requiredBytes <= this.getCacheFreeBytes();
    if (hasFreeSpace()) {
      return;
    }
    const cacheEntries = Object.entries(this.cache);
    for (const [cacheKey, cacheValue] of cacheEntries) {
      if (cacheValue === undefined) {
        continue;
      }
      await this.removeFromCache(cacheKey, cacheValue);
      if (hasFreeSpace()) {
        return;
      }
    }
  }
  getCacheFreeBytes() {
    return this.maxFontCacheSizeBytes - this.cacheSize;
  }
  async removeFromCache(cacheKey, cacheValue) {
    // eslint-disable-next-line @typescript-eslint/no-dynamic-delete
    delete this.cache[cacheKey];
    this.cacheSize -= cacheValue.localFileSize;
    await fs_1.promises.rm(cacheValue.localFilePath, {
      force: true
    });
    console.log(`Removed cached font: ${cacheValue.fontName}`);
  }
  getFileHash(fileBuffer) {
    const hashSum = crypto.createHash("sha256");
    hashSum.update(fileBuffer);
    return hashSum.digest("hex");
  }
}
exports.FontCachingService = FontCachingService;

/***/ }),

/***/ "./src/services/FontRenamingService.ts":
/*!*********************************************!*\
  !*** ./src/services/FontRenamingService.ts ***!
  \*********************************************/
/***/ ((__unused_webpack_module, exports, __webpack_require__) => {



Object.defineProperty(exports, "__esModule", ({
  value: true
}));
exports.FontRenamingService = void 0;
const fonteditor_core_1 = __webpack_require__(/*! fonteditor-core */ "fonteditor-core");
class FontRenamingService {
  rename(fontData, fontType, newFontName) {
    const font = fonteditor_core_1.Font.create(fontData, {
      type: fontType
    });
    const name = font.get().name;
    name.fontFamily = newFontName;
    name.ID = newFontName;
    name.fullName = newFontName;
    name.fontSubfamily = "Regular";
    name.postScriptName = `${name.fullName}-${name.fontSubfamily}`;
    return font.write({
      toBuffer: true,
      type: "ttf"
    });
  }
}
exports.FontRenamingService = FontRenamingService;

/***/ }),

/***/ "./src/services/LayerService.ts":
/*!**************************************!*\
  !*** ./src/services/LayerService.ts ***!
  \**************************************/
/***/ (function(__unused_webpack_module, exports, __webpack_require__) {



var __importDefault = this && this.__importDefault || function (mod) {
  return mod && mod.__esModule ? mod : {
    "default": mod
  };
};
Object.defineProperty(exports, "__esModule", ({
  value: true
}));
exports.LayerService = void 0;
const sharp_1 = __importDefault(__webpack_require__(/*! sharp */ "sharp"));
const TypeUtils_1 = __webpack_require__(/*! upload-image-plugin/common/TypeUtils */ "./src/common/TypeUtils.ts");
const LayerGravityMode_1 = __webpack_require__(/*! upload-image-plugin/params/LayerGravityMode */ "./src/params/LayerGravityMode.ts");
const LayerBlendMode_1 = __webpack_require__(/*! upload-image-plugin/params/LayerBlendMode */ "./src/params/LayerBlendMode.ts");
const PaddingDefinition_1 = __webpack_require__(/*! upload-image-plugin/model/PaddingDefinition */ "./src/model/PaddingDefinition.ts");
/**
 * TODO:
 *  - 'layer-fit' parameter
 *    - Supports all our existing modes for 'fit' parameter.
 *    - Will allow us to stretch images, or cover images, within the internal dimensions of a layer.
 *    - Current behaviour is essentially 'layer-fit=max'.
 */
class LayerService {
  constructor(downloadService, globalSharpOptions) {
    this.downloadService = downloadService;
    this.globalSharpOptions = globalSharpOptions;
    this.transparent = {
      alpha: 0,
      b: 0,
      g: 0,
      r: 0
    };
    this.sharpGravity = {
      "center": "center",
      "top": "north",
      "top-left": "northwest",
      "top-right": "northeast",
      "bottom": "south",
      "bottom-left": "southwest",
      "bottom-right": "southeast",
      "left": "west",
      "right": "east"
    };
    this.compositeBlendModeMapping = {
      "color-burn": "colour-burn",
      "color-dodge": "colour-dodge",
      "dest-atop": "dest-atop",
      "dest-in": "dest-in",
      "dest-out": "dest-out",
      "dest-over": "dest-over",
      "hard-light": "hard-light",
      "soft-light": "soft-light",
      "source-atop": "atop",
      "source-in": "in",
      "source-out": "out",
      "source-over": "over",
      "add": "add",
      "darken": "darken",
      "difference": "difference",
      "exclusion": "exclusion",
      "lighten": "lighten",
      "multiply": "multiply",
      "overlay": "overlay",
      "saturate": "saturate",
      "screen": "screen",
      "xor": "xor"
    };
    this.hAlign = {
      left: "start",
      center: "center",
      right: "end"
    };
    this.vAlign = {
      top: "start",
      center: "center",
      bottom: "end"
    };
  }
  async addLayer(img, input, step, eagerWork) {
    // Flush the existing pipeline (to apply transformations thus far) and also capture its dimensions, which are required for layering.
    const {
      data: baseLayerBuffer,
      info: baseLayerInfo
    } = await img.raw().toBuffer({
      resolveWithObject: true
    });
    const baseLayer = (0, sharp_1.default)(baseLayerBuffer, {
      raw: baseLayerInfo,
      ...this.globalSharpOptions
    });
    const {
      layer,
      layerSize
    } = await this.createLayer(baseLayerInfo, input, step, eagerWork);
    const {
      data: layerBuffer,
      info: layerRaw
    } = await (await this.handleCommonLayerSteps(baseLayerInfo, layer, layerSize, step)).raw().toBuffer({
      resolveWithObject: true
    });
    const absDefault = step.left === undefined && step.top === undefined ? undefined : 0;
    return baseLayer.composite([{
      input: layerBuffer,
      raw: layerRaw,
      top: step.top ?? absDefault,
      left: step.left ?? absDefault,
      tile: step.tile,
      premultiplied: !(step.premultiplyBaseImage ?? true),
      gravity: step.gravity === undefined ? undefined : this.sharpGravity[LayerGravityMode_1.LayerGravityMode.toString(step.gravity)],
      blend: step.blend === undefined ? undefined : this.compositeBlendModeMapping[LayerBlendMode_1.LayerBlendMode.toString(step.blend)]
    }]);
  }
  /**
   * Returns a sharp pipeline with no image format.
   */
  async createLayer(baseLayerInfo, input, step, eagerWork) {
    switch (step.type) {
      case "composite":
      case "layer":
        return await this.createImageLayer(baseLayerInfo, input, step);
      case "text":
        return await this.createTextLayer(baseLayerInfo, step, eagerWork);
      default:
        (0, TypeUtils_1.assertUnreachable)(step);
    }
  }
  async handleCommonLayerSteps(baseLayerSize, layer, layerSize, step) {
    let result = layer;
    let layerSizeUpdated = layerSize;
    if (step.opacity !== undefined || step.rotate !== undefined) {
      const {
        data,
        info
      } = await result.raw().toBuffer({
        resolveWithObject: true
      });
      result = (0, sharp_1.default)(data, {
        raw: info,
        ...this.globalSharpOptions
      });
    }
    if (step.opacity !== undefined) {
      result = result.composite([this.createOpacityLayer(step.opacity / 100)]);
    }
    if (step.rotate !== undefined) {
      result = result.rotate(step.rotate, {
        background: this.transparent
      });
      const {
        data,
        info
      } = await result.raw().toBuffer({
        resolveWithObject: true
      });
      result = (0, sharp_1.default)(data, {
        raw: info,
        ...this.globalSharpOptions
      });
      layerSizeUpdated = info;
    }
    result = this.maybeCropFromCenter(result, layerSizeUpdated, baseLayerSize);
    return result;
  }
  /**
   * Returns a sharp pipeline with no image format (for a text overlay).
   */
  async createTextLayer(baseLayerInfo, step, eagerWork) {
    const padding = this.getPadding(step);
    const fontSize = this.getFontSize(step);
    const layerMaxSize = this.getLayerMaxDimensions(baseLayerInfo, step);
    const font = step.fontFilePath !== undefined ? await eagerWork.fontFamilies[step.fontFilePath] :
    // : step.fontName !== undefined
    // ? {
    //     fontName: step.fontName,
    //     localFilePath: undefined
    //   }
    undefined;
    const text = {
      // Pango Markup: https://docs.gtk.org/Pango/pango_markup.html
      text: `<span ${this.createTextAttributes(step)}>${step.text}</span>`,
      font: font?.fontName,
      fontfile: font?.localFilePath,
      align: step.align,
      justify: step.justify,
      spacing: step.lineHeight,
      rgba: true,
      ...this.calculateFontSize(layerMaxSize, padding, fontSize)
    };
    const inputSize = await this.getDimensionsFromLayer((0, sharp_1.default)({
      text
    }));
    return await this.wrapInInvisibleBox(inputSize, {
      text
    }, undefined, step, padding);
  }
  /**
   * Returns a sharp pipeline with no image format (for an image overlay).
   */
  async createImageLayer(baseLayerInfo, input, step) {
    const padding = this.getPadding(step);
    const layerMaxSize = this.getLayerMaxDimensions(baseLayerInfo, step);
    const internalDimensions = this.calculateInternalDimensions(layerMaxSize, padding);
    const layer = (0, sharp_1.default)(this.downloadService.getTempDownloadedFile(input.tempDownloadedFiles, this.downloadService.makeLocalFileNameForLayerFileSource(step)), this.globalSharpOptions).resize({
      width: internalDimensions.width,
      height: internalDimensions.height,
      fit: "inside"
    });
    const {
      data,
      info
    } = await layer.raw().toBuffer({
      resolveWithObject: true
    });
    return await this.wrapInInvisibleBox(info, data, info, step, padding);
  }
  maybeCropFromCenter(layer, currentSize, maxSize) {
    const targetSize = {
      width: Math.min(currentSize.width, maxSize.width),
      height: Math.min(currentSize.height, maxSize.height)
    };
    const exceedsTargetSize = targetSize.width < currentSize.width || targetSize.height < currentSize.height;
    return exceedsTargetSize ? this.cropFromCenter(layer, currentSize, targetSize) : layer;
  }
  cropFromCenter(layer, currentSize, targetSize) {
    return layer.extract({
      top: Math.round(currentSize.height / 2 - targetSize.height / 2),
      left: Math.round(currentSize.width / 2 - targetSize.width / 2),
      height: targetSize.height,
      width: targetSize.width
    });
  }
  createOpacityLayer(opacityFactor) {
    return {
      input: Buffer.from([0, 0, 0, Math.round(255 * opacityFactor)]),
      raw: {
        width: 1,
        height: 1,
        channels: 4
      },
      tile: true,
      blend: "dest-in"
    };
  }
  async getDimensionsFromLayer(image) {
    const metadata = await image.metadata();
    return {
      width: metadata.width ?? 0,
      height: metadata.height ?? 0
    };
  }
  /**
   * Gets the maximum dimensions of the layer.
   */
  getLayerMaxDimensions(baseLayerInfo, step) {
    return {
      width: step.width ?? Math.max(0, baseLayerInfo.width - (step.left ?? 0)),
      height: step.height ?? Math.max(0, baseLayerInfo.height - (step.top ?? 0))
    };
  }
  getFontSize(step) {
    return step.fontSize ?? {
      type: "automatic"
    };
  }
  getPadding(step) {
    return PaddingDefinition_1.PaddingDefinition.getPadding(PaddingDefinition_1.PaddingDefinition.fromLayerStep(step));
  }
  calculateFontSize(layerMaxDimensions, padding, fontSize) {
    const internalDimensions = this.calculateInternalDimensions(layerMaxDimensions, padding);
    if (fontSize.type === "automatic") {
      return internalDimensions;
    }
    return {
      width: internalDimensions.width,
      dpi: fontSize.dpi // 'dpi' and 'height' are mutually exclusive.
    };
  }

  calculateInternalDimensions(layerMaxDimensions, padding) {
    const calculateDimension = (dimension, padding) => Math.max(0, dimension - padding);
    return {
      width: calculateDimension(layerMaxDimensions.width, this.getPaddingSize(padding, "x")),
      height: calculateDimension(layerMaxDimensions.height, this.getPaddingSize(padding, "y"))
    };
  }
  getPaddingSize(padding, dimension) {
    return dimension === "x" ? padding.left + padding.right : padding.top + padding.bottom;
  }
  async wrapInInvisibleBox(inputSize, input, inputRaw, step, padding) {
    const inputSizeWithPadding = {
      width: inputSize.width + this.getPaddingSize(padding, "x"),
      height: inputSize.height + this.getPaddingSize(padding, "y")
    };
    // If the layer's width/height was not explicitly set, take the width of the text.
    // If the layer's width/height were explicitly set, but the text overflows them, take the text's dimensions.
    const layerSize = {
      width: Math.max(step.width ?? 0, inputSizeWithPadding.width),
      height: Math.max(step.height ?? 0, inputSizeWithPadding.height)
    };
    const layer = this.addInputToLayer((0, sharp_1.default)({
      create: {
        ...layerSize,
        channels: 4,
        background: this.transparent
      }
    }), input, inputRaw, layerSize, inputSize, step, padding);
    return {
      layer,
      layerSize
    };
  }
  addInputToLayer(outerLayer, innerLayer, innerLayerRaw, outerLayerSize, innerLayerSize, step, padding) {
    const points = this.calculateInnerLayerOffset(outerLayerSize, innerLayerSize, step, padding);
    return outerLayer.composite([{
      input: innerLayer,
      raw: innerLayerRaw,
      top: points.y,
      left: points.x
    }]);
  }
  calculateInnerLayerOffset(outerLayerSize, innerLayerSize, step, padding) {
    return {
      x: this.align(outerLayerSize.width, innerLayerSize.width, this.hAlign[step.align ?? "center"], padding.left, padding.right),
      y: this.align(outerLayerSize.height, innerLayerSize.height, this.vAlign[step.valign ?? "center"], padding.top, padding.bottom)
    };
  }
  align(outerLayerSize, innerLayerSize, align, paddingStart, paddingEnd) {
    switch (align) {
      case "start":
        return paddingStart;
      case "end":
        return outerLayerSize - (innerLayerSize + paddingEnd);
      case "center":
        return paddingStart + Math.round((outerLayerSize - (paddingStart + paddingEnd)) / 2 - innerLayerSize / 2);
      default:
        (0, TypeUtils_1.assertUnreachable)(align);
    }
  }
  createTextAttributes(step) {
    return `color="${step.color}" ${step.fontWeight !== undefined ? `weight="${step.fontWeight}"` : ""} style="${step.fontStyle}" ${this.createUnderlineAttributes(step)} ${step.letterSpacing !== undefined ? `letter_spacing="${step.letterSpacing}"` : ""}`;
  }
  createUnderlineAttributes(step) {
    if (step.underline === undefined) {
      return "";
    }
    switch (step.underline.type) {
      case "single":
      case "double":
      case "error":
      case "low":
        return `underline="${step.underline.type}" ${step.underline.color !== undefined ? `underline_color="${step.underline.color}"` : ""}`;
      default:
        (0, TypeUtils_1.assertUnreachable)(step.underline);
    }
  }
}
exports.LayerService = LayerService;

/***/ }),

/***/ "./src/services/MathService.ts":
/*!*************************************!*\
  !*** ./src/services/MathService.ts ***!
  \*************************************/
/***/ ((__unused_webpack_module, exports) => {



Object.defineProperty(exports, "__esModule", ({
  value: true
}));
exports.MathService = void 0;
class MathService {
  integerRange(min, max, factor) {
    const range = max - min;
    const rangeFactored = Math.round(range * factor);
    return min + rangeFactored;
  }
  /**
   * @param percentage From 1 to 100.
   */
  percentageToGaussianSigma(percentage, max) {
    const min = 0.3;
    const range = max - min;
    // This converts the range 1-100 to 0-100. This is because we advertise a minimum blur of 1 (which we want to
    // map to Sharp's minimum blur of 0.3) rather than advertise a minimum blur of 0, which would be confusing as it
    // would misleadingly imply no blur.
    const zeroToHundred = (percentage - 1) * (100 / 99);
    return min + range * (zeroToHundred / 100);
  }
  /**
   * Converts a value on an absolute range to a value on a relative range:
   * - If the input value is  < 0, then the value is converted to a range 0..1
   * - If the input value is >= 0, then the value is converted to a range 1..X
   * Example:
   * - Given Ranges: -100...0...100  |  0.1...1...10
   * - F(-100)       = 0.1
   * - F(0)          = 1
   * - F(100)        = 10
   */
  toRelativeValue(absoluteValue, absoluteRange, relativeRange) {
    if (absoluteValue >= 0) {
      return this.selectOnRange(absoluteValue / absoluteRange.max, 1, relativeRange.max);
    }
    return this.selectOnRange(Math.abs(absoluteValue) / Math.abs(absoluteRange.min), 1, relativeRange.min);
  }
  selectOnRange(factor, min, max) {
    const range = Math.abs(min - max);
    const progress = factor * range;
    if (min < max) {
      return min + progress;
    }
    return min - progress;
  }
}
exports.MathService = MathService;

/***/ }),

/***/ "./src/services/PipelineExecutionService.ts":
/*!**************************************************!*\
  !*** ./src/services/PipelineExecutionService.ts ***!
  \**************************************************/
/***/ (function(__unused_webpack_module, exports, __webpack_require__) {



var __importDefault = this && this.__importDefault || function (mod) {
  return mod && mod.__esModule ? mod : {
    "default": mod
  };
};
Object.defineProperty(exports, "__esModule", ({
  value: true
}));
exports.PipelineExecutionService = void 0;
const sharp_1 = __importDefault(__webpack_require__(/*! sharp */ "sharp"));
const TypeUtils_1 = __webpack_require__(/*! upload-image-plugin/common/TypeUtils */ "./src/common/TypeUtils.ts");
const OutputQuality_1 = __webpack_require__(/*! upload-image-plugin/params/OutputQuality */ "./src/params/OutputQuality.ts");
const Compression_1 = __webpack_require__(/*! upload-image-plugin/params/Compression */ "./src/params/Compression.ts");
const PaddingDefinition_1 = __webpack_require__(/*! upload-image-plugin/model/PaddingDefinition */ "./src/model/PaddingDefinition.ts");
class PipelineExecutionService {
  constructor(layerService, mathService, globalSharpOptions) {
    this.layerService = layerService;
    this.mathService = mathService;
    this.globalSharpOptions = globalSharpOptions;
  }
  async executePipeline(input, steps, outputFormat, pages, eagerWork) {
    const inputResolved = input.path;
    let image = (0, sharp_1.default)(inputResolved, {
      ...this.globalSharpOptions,
      page: pages.page,
      pages: pages.pageCount
    });
    for (const step of steps) {
      image = await this.executeStep(input, step, image, eagerWork);
    }
    image = image.toFormat(outputFormat.type);
    return this.setQuality(image, outputFormat, input);
  }
  async executeStep(input, step, img, eagerWork) {
    switch (step.type) {
      case "crop":
        return img.extract(this.getCropOptions(step.geometry));
      case "resize":
        return img.resize(this.getResizeOptions(step.geometry.size));
      case "negative":
        return img.negate({
          alpha: step.negateAlpha
        });
      case "grayscale":
        return img.grayscale(true);
      case "tint":
        return img.tint(step.color);
      case "modulate":
        {
          const inputRangeMax = 100; // Max value the user provides
          const toRelative = (value, min, max) => this.mathService.toRelativeValue(value, {
            min: inputRangeMax * -1,
            max: inputRangeMax
          }, {
            min,
            max
          });
          return img.modulate({
            hue: step.hue,
            lightness: step.lightness * (255 / 100),
            saturation: toRelative(step.saturation, 0.01, 6),
            brightness: toRelative(step.brightness, 0.01, 6)
          });
        }
      case "flip":
        {
          switch (step.axis.type) {
            case "x":
              return img.flop();
            case "y":
              return img.flip();
            default:
              (0, TypeUtils_1.assertUnreachable)(step.axis);
          }
          break;
        }
      case "rotate":
        {
          switch (step.mode.type) {
            case "automatic":
              return img.rotate();
            case "manual":
              return img.rotate(step.mode.degrees, {
                background: step.mode.backgroundColor
              });
            default:
              (0, TypeUtils_1.assertUnreachable)(step.mode);
          }
          break;
        }
      case "sharpen":
        {
          switch (step.mode.type) {
            case "fast":
              return img.sharpen();
            case "manual":
              return img.sharpen(this.mathService.percentageToGaussianSigma(step.mode.percentage, 20));
            // 1000 times out (and the image doesn't actually sharpen beyond a certain amount).
            default:
              (0, TypeUtils_1.assertUnreachable)(step.mode);
          }
          break;
        }
      case "blur":
        {
          switch (step.mode.type) {
            case "fast":
              return img.blur();
            case "manual":
              return img.blur(this.mathService.percentageToGaussianSigma(step.mode.percentage, 100));
            // 1000 times out (& is far too blurry).
            default:
              (0, TypeUtils_1.assertUnreachable)(step.mode);
          }
          break;
        }
      case "extend":
        {
          const options = {
            ...PaddingDefinition_1.PaddingDefinition.getPadding(PaddingDefinition_1.PaddingDefinition.fromExtendStep(step)),
            background: step.extendColor
          };
          return img.extend(options);
        }
      case "composite":
      case "layer":
      case "text":
        {
          return await this.layerService.addLayer(img, input, step, eagerWork);
        }
      case "flush":
        {
          const {
            data,
            info
          } = await img.raw().toBuffer({
            resolveWithObject: true
          });
          return (0, sharp_1.default)(data, {
            raw: info,
            ...this.globalSharpOptions
          });
        }
      default:
        (0, TypeUtils_1.assertUnreachable)(step);
    }
  }
  setQuality(img, format, input) {
    const quality = input.pipeline.outputQuality ?? OutputQuality_1.OutputQuality.defaultValue;
    const compression = input.pipeline.compression ?? Compression_1.Compression.defaultValue;
    switch (format.type) {
      case "jpeg":
      case "jpg":
        return img.jpeg({
          quality
        });
      case "png":
        return img.png({
          quality,
          compressionLevel: this.mathService.integerRange(0, 9, compression / 100),
          palette: true,
          dither: 0 // 11% reduction in processing time, 4% reduction in file size, not much visual difference.
        });

      case "gif":
        return img.gif({
          // We use 100% quality (i.e. all colours) by default for GIFs.
          colors: this.mathService.integerRange(2, 256, (input.pipeline.outputQuality ?? 100) / 100),
          effort: this.mathService.integerRange(1, 10, compression / 100),
          reoptimise: true,
          dither: 0
        });
      case "webp":
        return img.webp({
          quality,
          // lossless: quality === 100, // Results in timeouts (operation takes too long).
          alphaQuality: quality,
          effort: this.mathService.integerRange(0, 6, compression / 100)
        });
      case "avif":
        return img.avif({
          quality,
          effort: this.mathService.integerRange(0, 9, compression / 100)
        });
    }
  }
  getCropOptions({
    size,
    offset
  }) {
    return {
      width: size.width,
      height: size.height,
      top: Math.max(0, offset.y),
      left: Math.max(0, offset.x)
    };
  }
  getResizeOptions(size) {
    switch (size.type) {
      case "xheight":
        return {
          height: size.height
        };
      case "width":
        return {
          width: size.width
        };
      case "widthxheight":
        return {
          width: size.width,
          height: size.height,
          fit: "inside"
        };
      case "widthxheight^":
        return {
          width: size.width,
          height: size.height,
          fit: "outside"
        };
      case "widthxheight!":
        return {
          width: size.width,
          height: size.height,
          fit: "fill"
        };
      case "widthxheight>":
        return {
          width: size.width,
          height: size.height,
          fit: "inside",
          withoutEnlargement: true
        };
      case "widthxheight>cover":
        return {
          width: size.width,
          height: size.height,
          fit: "outside",
          withoutEnlargement: true
        };
      case "widthxheight<":
        return {
          width: size.width,
          height: size.height,
          fit: "inside",
          withoutReduction: true
        };
      case "widthxheight<cover":
        return {
          width: size.width,
          height: size.height,
          fit: "outside",
          withoutReduction: true
        };
      case "widthxheightc":
        return {
          width: size.width,
          height: size.height,
          fit: "cover",
          position: this.getCropPosition(size.cropStrategy)
        };
      default:
        (0, TypeUtils_1.assertUnreachable)(size);
    }
  }
  getCropPosition(cropStrategy) {
    const strategies = {
      fixedBottom: "bottom",
      fixedBottomLeft: "left bottom",
      fixedBottomRight: "right bottom",
      fixedCenter: "center",
      fixedLeft: "left",
      fixedRight: "right",
      fixedTop: "top",
      fixedTopLeft: "left top",
      fixedTopRight: "right top",
      smartAttention: "attention",
      smartEntropy: "entropy"
    };
    return strategies[cropStrategy.type];
  }
}
exports.PipelineExecutionService = PipelineExecutionService;

/***/ }),

/***/ "./src/services/PipelineLintingService.ts":
/*!************************************************!*\
  !*** ./src/services/PipelineLintingService.ts ***!
  \************************************************/
/***/ ((__unused_webpack_module, exports) => {



Object.defineProperty(exports, "__esModule", ({
  value: true
}));
exports.PipelineLintingService = void 0;
/**
 * Responsible for adding default steps to a pipeline, and removing nonsensical steps.
 */
class PipelineLintingService {
  constructor() {
    this.flushStep = {
      type: "flush"
    };
    this.requiresFlushBefore = new Set([]);
    this.requiresFlushAfter = new Set(["layer", "composite", "text"]);
    /**
     * The steps automatically flush after they're run.
     */
    this.autoPostFlushingSteps = new Set([]);
    /**
     * The steps automatically flush before they're run.
     */
    this.autoPreFlushingSteps = new Set(["layer", "composite", "text"]);
  }
  lintPipeline(steps) {
    // TODO: remove rotates that have 0 or 360 _before_ the 'add default rotation' step.
    // TODO: set 'rotate' to undefined if '0' or '360' in all LayerBaseStep.
    return this.addFlushes(this.addDefaultRotation(steps));
  }
  /**
   * Automatically flushes the pipeline when steps of the same type are found, e.g. 2x rotations, as Sharp will
   * essentially use a "last step wins" approach by default.
   */
  addFlushes(steps) {
    const flushableSteps = new Set([]);
    let flushBeforeNextStep = false;
    const onFlush = () => {
      flushBeforeNextStep = false;
      flushableSteps.clear();
    };
    return steps.flatMap(step => {
      const hasFlushableSteps = flushableSteps.size > 0;
      // Handle explicit flush steps by removing them when unnecessary.
      if (step.type === "flush") {
        if (hasFlushableSteps) {
          onFlush();
          return [this.flushStep];
        }
        return [];
      }
      const isDuplicateStepType = flushableSteps.has(step.type); // Always flush between two of the same steps, e.g. two rotates, two tints, etc.
      const forceFlushBeforeCurrentStep = this.requiresFlushBefore.has(step.type);
      const forceFlushAfterCurrentStep = this.requiresFlushAfter.has(step.type);
      const hasAutoPreFlush = this.autoPreFlushingSteps.has(step.type);
      const hasAutoPostFlush = this.autoPostFlushingSteps.has(step.type);
      const flushBeforeCurrentStep = hasFlushableSteps && !hasAutoPreFlush && (flushBeforeNextStep || forceFlushBeforeCurrentStep || isDuplicateStepType);
      if (hasAutoPreFlush || flushBeforeCurrentStep) {
        onFlush();
      }
      flushableSteps.add(step.type);
      if (forceFlushAfterCurrentStep) {
        // Flush before and after steps of type 'alwaysFlush', e.g. if running a 'layer' and then a 'blur' or a 'tint',
        // in both cases we want the 'layer' flushed before the subsequent step is applied.
        flushBeforeNextStep = true;
      }
      if (hasAutoPostFlush) {
        // Must go after 'flushableSteps.add' as we need to remove ourselves from the flushable steps
        onFlush();
      }
      return flushBeforeCurrentStep ? [this.flushStep, step] : [step];
    });
  }
  /**
   * Automatically adds a rotate step if non exists, effectively rotating the image by its EXIF rotation metadata (if
   * any exists) so that the EXIF rotation metadata can be deleted (which is what we do on save).
   */
  addDefaultRotation(steps) {
    const hasRotation = steps.some(x => x.type === "rotate");
    const defaultRotation = {
      mode: {
        type: "automatic"
      },
      type: "rotate"
    };
    return hasRotation ? steps : [defaultRotation, ...steps];
  }
}
exports.PipelineLintingService = PipelineLintingService;

/***/ }),

/***/ "./src/services/PipelineMergingService.ts":
/*!************************************************!*\
  !*** ./src/services/PipelineMergingService.ts ***!
  \************************************************/
/***/ ((__unused_webpack_module, exports, __webpack_require__) => {



Object.defineProperty(exports, "__esModule", ({
  value: true
}));
exports.PipelineMergingService = void 0;
const ImagePipelineMergeBehaviour_1 = __webpack_require__(/*! upload-image-plugin/params/ImagePipelineMergeBehaviour */ "./src/params/ImagePipelineMergeBehaviour.ts");
const TypeUtils_1 = __webpack_require__(/*! upload-image-plugin/common/TypeUtils */ "./src/common/TypeUtils.ts");
const ramda_1 = __webpack_require__(/*! ramda */ "ramda");
class PipelineMergingService {
  mergePipelines(file, master) {
    const behaviour = master.pipelineMergeBehaviour ?? ImagePipelineMergeBehaviour_1.ImagePipelineMergeBehaviour.defaultValue;
    return {
      outputFormat: this.mergeScalar(file.pipeline, master.pipeline, behaviour, "outputFormat"),
      outputQuality: this.mergeScalar(file.pipeline, master.pipeline, behaviour, "outputQuality"),
      compression: this.mergeScalar(file.pipeline, master.pipeline, behaviour, "compression"),
      pages: master.pipeline.pages,
      steps: this.mergeSteps(file.pipeline, master.pipeline, behaviour)
    };
  }
  mergeScalar(file, master, behaviour, key) {
    const b = behaviour[key];
    switch (b) {
      case "master":
        return master[key];
      case "file":
        return file[key];
      case "fileThenMaster":
        return file[key] ?? master[key];
      default:
        (0, TypeUtils_1.assertUnreachable)(b);
    }
  }
  mergeSteps(file, master, behaviour) {
    if (behaviour.steps.mergeBehaviour === "master") {
      return master.steps;
    }
    const whitelist = behaviour.steps.fileWhitelist;
    const fileSteps = file.steps.filter(x => whitelist.includes(x.type));
    if (behaviour.steps.mergeBehaviour === "file") {
      return fileSteps;
    }
    let steps;
    switch (behaviour.steps.mergeBehaviour.startWith) {
      case "master":
        steps = [...master.steps, ...file.steps];
        break;
      case "file":
        steps = [...file.steps, ...master.steps];
        break;
      default:
        (0, TypeUtils_1.assertUnreachable)(behaviour.steps.mergeBehaviour.startWith);
    }
    switch (behaviour.steps.mergeBehaviour.removeDuplicates) {
      case "start":
        steps = (0, ramda_1.uniqBy)(x => x.type, steps);
        break;
      case "end":
        steps = (0, ramda_1.reverse)((0, ramda_1.uniqBy)(x => x.type, (0, ramda_1.reverse)(steps)));
        break;
      case false:
        break;
      default:
        (0, TypeUtils_1.assertUnreachable)(behaviour.steps.mergeBehaviour.removeDuplicates);
    }
    return steps;
  }
}
exports.PipelineMergingService = PipelineMergingService;

/***/ }),

/***/ "fonteditor-core":
/*!**********************************!*\
  !*** external "fonteditor-core" ***!
  \**********************************/
/***/ ((module) => {

module.exports = require("fonteditor-core");

/***/ }),

/***/ "ramda":
/*!************************!*\
  !*** external "ramda" ***!
  \************************/
/***/ ((module) => {

module.exports = require("ramda");

/***/ }),

/***/ "sharp":
/*!************************!*\
  !*** external "sharp" ***!
  \************************/
/***/ ((module) => {

module.exports = require("sharp");

/***/ }),

/***/ "upload-plugin-sdk":
/*!************************************!*\
  !*** external "upload-plugin-sdk" ***!
  \************************************/
/***/ ((module) => {

module.exports = require("upload-plugin-sdk");

/***/ }),

/***/ "crypto":
/*!*************************!*\
  !*** external "crypto" ***!
  \*************************/
/***/ ((module) => {

module.exports = require("crypto");

/***/ }),

/***/ "fs":
/*!*********************!*\
  !*** external "fs" ***!
  \*********************/
/***/ ((module) => {

module.exports = require("fs");

/***/ }),

/***/ "path":
/*!***********************!*\
  !*** external "path" ***!
  \***********************/
/***/ ((module) => {

module.exports = require("path");

/***/ })

/******/ 	});
/************************************************************************/
/******/ 	// The module cache
/******/ 	var __webpack_module_cache__ = {};
/******/ 	
/******/ 	// The require function
/******/ 	function __webpack_require__(moduleId) {
/******/ 		// Check if module is in cache
/******/ 		var cachedModule = __webpack_module_cache__[moduleId];
/******/ 		if (cachedModule !== undefined) {
/******/ 			return cachedModule.exports;
/******/ 		}
/******/ 		// Create a new module (and put it into the cache)
/******/ 		var module = __webpack_module_cache__[moduleId] = {
/******/ 			// no module.id needed
/******/ 			// no module.loaded needed
/******/ 			exports: {}
/******/ 		};
/******/ 	
/******/ 		// Execute the module function
/******/ 		__webpack_modules__[moduleId].call(module.exports, module, module.exports, __webpack_require__);
/******/ 	
/******/ 		// Return the exports of the module
/******/ 		return module.exports;
/******/ 	}
/******/ 	
/************************************************************************/
/******/ 	
/******/ 	// startup
/******/ 	// Load entry module and return exports
/******/ 	// This entry module is referenced by other modules so it can't be inlined
/******/ 	var __webpack_exports__ = __webpack_require__("./src/index.ts");
/******/ 	__webpack_exports__ = __webpack_exports__["default"];
/******/ 	
/******/ 	return __webpack_exports__;
/******/ })()
;
});