export declare type LayerBlendMode = LayerBlendModeString | LayerBlendModeObject;
export declare type LayerBlendModeString = "source-over" | "source-in" | "source-out" | "source-atop" | "dest-over" | "dest-in" | "dest-out" | "dest-atop" | "xor" | "add" | "saturate" | "multiply" | "screen" | "overlay" | "darken" | "lighten" | "color-dodge" | "color-burn" | "hard-light" | "soft-light" | "difference" | "exclusion";
export declare type LayerBlendModeObject = {
    /**
     * Blend Mode
     *
     * @description
     * Query Param: blend
     * Query Param Title: Layer Blend Mode
     *
     * Renders the layer over the base image.
     *
     * *Default blend mode.*
     */
    type: "source-over";
} | {
    /**
     * Blend Mode
     *
     * @description
     * Query Param: blend
     * Query Param Title: Layer Blend Mode
     *
     * Renders the layer only.
     *
     * The base image's alpha channel is used as a mask for the layer.
     */
    type: "source-in";
} | {
    /**
     * Blend Mode
     *
     * @description
     * Query Param: blend
     * Query Param Title: Layer Blend Mode
     *
     * Renders the layer only.
     *
     * The base image's alpha channel is used as an inverse mask for the layer.
     */
    type: "source-out";
} | {
    /**
     * Blend Mode
     *
     * @description
     * Query Param: blend
     * Query Param Title: Layer Blend Mode
     *
     * Renders the layer over the base image.
     *
     * Uses the base image's alpha channel for the layer's alpha channel.
     */
    type: "source-atop";
} | {
    /**
     * Blend Mode
     *
     * @description
     * Query Param: blend
     * Query Param Title: Layer Blend Mode
     *
     * Renders the base image over the layer.
     *
     * Opposite of `source-over`.
     */
    type: "dest-over";
} | {
    /**
     * Blend Mode
     *
     * @description
     * Query Param: blend
     * Query Param Title: Layer Blend Mode
     *
     * Renders the base image only.
     *
     * The layer's alpha channel is used as a mask for the base image.
     *
     * Opposite of `source-in`.
     */
    type: "dest-in";
} | {
    /**
     * Blend Mode
     *
     * @description
     * Query Param: blend
     * Query Param Title: Layer Blend Mode
     *
     * Renders the base image only.
     *
     * The layer's alpha channel is used as an inverse mask for the base image.
     *
     * Opposite of `source-out`.
     */
    type: "dest-out";
} | {
    /**
     * Blend Mode
     *
     * @description
     * Query Param: blend
     * Query Param Title: Layer Blend Mode
     *
     * Renders the base image over the layer.
     *
     * Uses the layer's alpha channel for the base image's alpha channel.
     *
     * Opposite of `source-atop`.
     */
    type: "dest-atop";
} | {
    /**
     * Blend Mode
     *
     * @description
     * Query Param: blend
     * Query Param Title: Layer Blend Mode
     *
     * Performs a XOR of the alpha channels between both layers.
     */
    type: "xor";
} | {
    /**
     * Blend Mode
     *
     * @description
     * Query Param: blend
     * Query Param Title: Layer Blend Mode
     *
     * Renders the sum of each pixel.
     */
    type: "add";
} | {
    /**
     * Blend Mode
     *
     * @description
     * Query Param: blend
     * Query Param Title: Layer Blend Mode
     *
     * Renders the base image over the layer.
     *
     * Darkens the layer per the layer's alpha channel (0% alpha = 100% darkness applied, 100% alpha = 0% darkness applied).
     */
    type: "saturate";
} | {
    /**
     * Blend Mode
     *
     * @description
     * Query Param: blend
     * Query Param Title: Layer Blend Mode
     *
     * Renders the layer over the base image.
     *
     * Intersecting pixels will be as dark as, or darker than, the original pixels from each.
     */
    type: "multiply";
} | {
    /**
     * Blend Mode
     *
     * @description
     * Query Param: blend
     * Query Param Title: Layer Blend Mode
     *
     * Renders the layer over the base image.
     *
     * Intersecting pixels will be as light as, or lighter than, the original pixels from each.
     */
    type: "screen";
} | {
    /**
     * Blend Mode
     *
     * @description
     * Query Param: blend
     * Query Param Title: Layer Blend Mode
     *
     * Renders the layer over the base image.
     *
     * Intersecting pixels are blended with `multiply` or `screen`, depending on the lightness of the layer's colors.
     */
    type: "overlay";
} | {
    /**
     * Blend Mode
     *
     * @description
     * Query Param: blend
     * Query Param Title: Layer Blend Mode
     *
     * Renders the layer over the base image.
     *
     * Intersecting pixels are mixed using the darkest color component from each layer.
     */
    type: "darken";
} | {
    /**
     * Blend Mode
     *
     * @description
     * Query Param: blend
     * Query Param Title: Layer Blend Mode
     *
     * Renders the layer over the base image.
     *
     * Intersecting pixels are mixed using the lightest color component from each layer.
     */
    type: "lighten";
} | {
    /**
     * Blend Mode
     *
     * @description
     * Query Param: blend
     * Query Param Title: Layer Blend Mode
     *
     * Renders the layer over the base image.
     *
     * Brightens the layer's color by a factor depending on the base layer's color.
     */
    type: "color-dodge";
} | {
    /**
     * Blend Mode
     *
     * @description
     * Query Param: blend
     * Query Param Title: Layer Blend Mode
     *
     * Renders the layer over the base image.
     *
     * Darkens the layer's color by a factor depending on the base layer's color.
     */
    type: "color-burn";
} | {
    /**
     * Blend Mode
     *
     * @description
     * Query Param: blend
     * Query Param Title: Layer Blend Mode
     *
     * Renders the layer over the base image.
     *
     * Intersecting pixels are blended with `multiply` or `screen`, depending on the lightness of the base image's colors.
     */
    type: "hard-light";
} | {
    /**
     * Blend Mode
     *
     * @description
     * Query Param: blend
     * Query Param Title: Layer Blend Mode
     *
     * Renders the layer over the base image.
     *
     * Intersecting pixels are blended with `darken` or `lighten`, depending on the lightness of the base image's colors.
     */
    type: "soft-light";
} | {
    /**
     * Blend Mode
     *
     * @description
     * Query Param: blend
     * Query Param Title: Layer Blend Mode
     *
     * Renders the difference of the two images.
     */
    type: "difference";
} | {
    /**
     * Blend Mode
     *
     * @description
     * Query Param: blend
     * Query Param Title: Layer Blend Mode
     *
     * Renders the difference of the two images (with a lower contrast than the `difference` blend mode).
     */
    type: "exclusion";
};
export declare namespace LayerBlendMode {
    function toString(format: LayerBlendMode): LayerBlendModeString;
}
