export declare type SupportedOutputFormat = SupportedOutputFormatString | SupportedOutputFormatObject;
/**
 * Legacy string-based formats, still used by transformations generated by the transformation preset builder UI.
 */
export declare type SupportedOutputFormatString = "gif" | "jpg" | "jpeg" | "png" | "webp" | "avif";
/**
 * These are used by the querystring API, and more generally, allow us to define per-format options in the future,
 * e.g. palette and animation settings for GIFs, etc.
 */
export declare type SupportedOutputFormatObject = {
    /**
     * @description
     * Query Param: f
     * Query Param Title: Format
     *
     * Converts the image to GIF.
     */
    type: "gif";
} | {
    /**
     * @description
     * Query Param: f
     * Query Param Title: Format
     *
     * Converts the image to JPEG.
     */
    type: "jpeg";
} | {
    /**
     * @description
     * Query Param: f
     * Query Param Title: Format
     *
     * Converts the image to JPEG. (Alias for `f=jpeg`.)
     */
    type: "jpg";
} | {
    /**
     * @description
     * Query Param: f
     * Query Param Title: Format
     *
     * Converts the image to PNG.
     */
    type: "png";
} | {
    /**
     * @description
     * Query Param: f
     * Query Param Title: Format
     *
     * Converts the image to WebP.
     */
    type: "webp";
} | {
    /**
     * @description
     * Query Param: f
     * Query Param Title: Format
     *
     * Converts the image to AVIF.
     */
    type: "avif";
};
export declare namespace SupportedOutputFormat {
    /**
     * Keyed by format, to make sure the compiler ensures we specify a mime type for all supported formats.
     */
    const contentTypeReverseMap: Record<SupportedOutputFormatString, string>;
    const defaultOutputFormatSettings: Record<SupportedOutputFormatString, SupportedOutputFormatObject>;
    function toObject(format: SupportedOutputFormat): SupportedOutputFormatObject;
    const contentTypes: [SupportedOutputFormat, string][];
}
