import { ColorRGB } from "../params/ColorRGB";
export declare type UnderlineMode = UnderlineSingle | UnderlineDouble | UnderlineLow | UnderlineError;
interface UnderlineSingle extends UnderlineModeBase {
    /**
     * Underline
     *
     * @description
     * Query Param: underline
     * Query Param Title: Text Underline
     * Query Param Precedence: 0
     *
     * Underlines the text with a single line.
     */
    type: "single";
}
interface UnderlineDouble extends UnderlineModeBase {
    /**
     * Underline
     *
     * @description
     * Query Param: underline
     * Query Param Title: Text Underline
     * Query Param Precedence: 1
     *
     * Underlines the text with a double line.
     */
    type: "double";
}
interface UnderlineLow extends UnderlineModeBase {
    /**
     * Underline
     *
     * @description
     * Query Param: underline
     * Query Param Title: Text Underline
     * Query Param Precedence: 2
     *
     * Underlines the text with a single line (closer to the text than `single`).
     */
    type: "low";
}
interface UnderlineError extends UnderlineModeBase {
    /**
     * Underline
     *
     * @description
     * Query Param: underline
     * Query Param Title: Text Underline
     * Query Param Precedence: 3
     *
     * Underlines the text with a single wiggly line, similar to the "spelling error" underline used by most word processing applications.
     */
    type: "error";
}
interface UnderlineModeBase {
    /**
     * Underline Color
     *
     * @description
     * Query Param: underline-color
     * Query Param Title: Text Underline Color
     *
     * Sets the color to use for the `underline` parameter.
     */
    color?: ColorRGB;
}
export {};
