import { FontInfo } from "../model/FontInfo";
import { FontType } from "../model/FontType";
import { FontRenamingService } from "../services/FontRenamingService";
export declare class FontCachingService {
    private readonly fontRenamingService;
    /**
     * We limit font sizes, as we cache them permanently in the Lambda's temp dir.
     * @private
     */
    private readonly maxFontSizeBytes;
    private readonly maxFontCacheSizeBytes;
    /**
     * Key: hash of the original font file
     */
    private cache;
    private cacheSize;
    private readonly fontDir;
    constructor(fontRenamingService: FontRenamingService);
    /**
     * Inspects the 'localFontPath' and determines if the font has been seen before, based on the file's hash.
     * - If the font HAS been seen before, returns an existing cached font file for it.
     * - If the font HAS NOT been seen before, then generates a reusable font file for it (that can be reused by
     *   subsequent transformations), and returns it.
     *
     * We do this because libvips/pango/cairo maintain an in-process cache fonts based off their embedded font name (not
     * the file name), so if we give libvips a font file path, delete it, then provide a different font path (for the same
     * font), then Pango will error, since it will attempt to load the font using its original file path, which no-longer
     * exists. The simplest workaround is to rename (the font family) of each font file before it's used. However, this
     * will cause the internal font cache inside libvips/pango to grow after each transformation, essentially resulting in
     * a memory leak. The other solution is to store the font files in a global directory (that is shared between
     * invocations), but this introduces a security issue whereby malicious users can install bad fonts that will be used
     * by other users. The best solution is to combine the approaches: store the font files in a global directory, but
     * rename the font families to use file's hash, preventing malicious users from installing bad fonts for other users.
     *
     * This method does exactly that (it installs the given font to a global font dir used by all transformations, and
     * renames the font's font family to the file's hash).
     */
    installCustomFont(tempFontFilePath: string, fontType: FontType): Promise<FontInfo | undefined>;
    private installNewFont;
    private addToCache;
    private freeUpCache;
    private getCacheFreeBytes;
    private removeFromCache;
    private getFileHash;
}
