import { Sharp, SharpOptions } from "sharp";
import { CompositeStep, LayerImageStep, LayerTextStep } from "../params/ImagePipelineStep";
import { DownloadService } from "../services/DownloadService";
import { TransformationInput } from "../model/TransformationInput";
import { EagerWork } from "../model/EagerWork";
declare type LayerStep = LayerImageStep | CompositeStep | LayerTextStep;
/**
 * TODO:
 *  - 'layer-fit' parameter
 *    - Supports all our existing modes for 'fit' parameter.
 *    - Will allow us to stretch images, or cover images, within the internal dimensions of a layer.
 *    - Current behaviour is essentially 'layer-fit=max'.
 */
export declare class LayerService {
    private readonly downloadService;
    private readonly globalSharpOptions;
    private readonly transparent;
    private readonly sharpGravity;
    private readonly compositeBlendModeMapping;
    private readonly hAlign;
    private readonly vAlign;
    constructor(downloadService: DownloadService, globalSharpOptions: SharpOptions);
    addLayer(img: Sharp, input: TransformationInput, step: LayerStep, eagerWork: EagerWork): Promise<Sharp>;
    /**
     * Returns a sharp pipeline with no image format.
     */
    private createLayer;
    private handleCommonLayerSteps;
    /**
     * Returns a sharp pipeline with no image format (for a text overlay).
     */
    private createTextLayer;
    /**
     * Returns a sharp pipeline with no image format (for an image overlay).
     */
    private createImageLayer;
    private maybeCropFromCenter;
    private cropFromCenter;
    private createOpacityLayer;
    private getDimensionsFromLayer;
    /**
     * Gets the maximum dimensions of the layer.
     */
    private getLayerMaxDimensions;
    private getFontSize;
    private getPadding;
    private calculateFontSize;
    private calculateInternalDimensions;
    private getPaddingSize;
    private wrapInInvisibleBox;
    private addInputToLayer;
    private calculateInnerLayerOffset;
    private align;
    private createTextAttributes;
    private createUnderlineAttributes;
}
export {};
