import { ImagePipelineStep } from "../params/ImagePipelineStep";
/**
 * Responsible for adding default steps to a pipeline, and removing nonsensical steps.
 */
export declare class PipelineLintingService {
    private readonly flushStep;
    private readonly requiresFlushBefore;
    private readonly requiresFlushAfter;
    /**
     * The steps automatically flush after they're run.
     */
    private readonly autoPostFlushingSteps;
    /**
     * The steps automatically flush before they're run.
     */
    private readonly autoPreFlushingSteps;
    lintPipeline(steps: ImagePipelineStep[]): ImagePipelineStep[];
    /**
     * Automatically flushes the pipeline when steps of the same type are found, e.g. 2x rotations, as Sharp will
     * essentially use a "last step wins" approach by default.
     */
    private addFlushes;
    /**
     * Automatically adds a rotate step if non exists, effectively rotating the image by its EXIF rotation metadata (if
     * any exists) so that the EXIF rotation metadata can be deleted (which is what we do on save).
     */
    private addDefaultRotation;
}
